==> cctda_0.0.43.tar.gz: cctda/R/get_ccsqlstr.R <==
#' Get SQL Conditional Statement for ccdb
#'
#' This function constructs an SQL WHERE clause based on provided parameters
#' for querying the cc database.
#'
#' @param daterange Date range as a vector of two dates.
#' @param datelim (deprecated!) Date range as a vector of two dates.
#' @param pids Project IDs as a string.
#' @param sites Sites as a string.
#' @param zlim Water depth range of samples as a numeric vector.
#' @param reps Repeat sample IDs as a string.
#' @param taxs Taxonomy names as a string.
#' @param extracond Extra condition statement as a string.
#'
#' @return A string representing the SQL WHERE clause.
#' @importFrom lubridate today
#' @importFrom stringr str_c str_split
#' @export
#'
get_ccsqlstr <- function(daterange = lubridate::today() - c(365, 0),
                         pids = "",
                         sites = "",
                         zlim = c(0, 30),
                         taxs = "",
                         reps = "",
                         extracond = "",
                         datelim = NULL) {
  if (!is.null(datelim)) {
    warning("'datelim' is deprecated; using 'daterange' instead!")
    daterange <- datelim
  }
  # Construct SQL conditions
  datestr <- sprintf("(`date` BETWEEN '%s' AND '%s')", daterange[1], daterange[2])
  pidstr <- pidsitesqlstr(pids, type = "pid")
  sitestr <- pidsitesqlstr(sites, type = "site")
  zstr <- sprintf("(`depth` >= %d AND `depth` <= %d)", zlim[1], zlim[2])
  taxstr <- taxsqlstr(taxs, tbname = "cc")
  repstr <- repsqlstr(reps)
  extracondstr <- extracondsqlstr(extracond)

  # Collect all conditions
  condstrs <- c(datestr, pidstr, sitestr, zstr, taxstr, repstr, extracondstr)
  condstrs <- condstrs[nzchar(condstrs)] # Remove empty conditions

  if (length(condstrs) == 0) {
    return("")
  }

  return(paste0(" WHERE (", paste(condstrs, collapse = " AND "), ")"))
}

#' Generate Extra Conditional SQL Statement
#'
#' This function formats an extra SQL condition.
#'
#' @param extracond Custom SQL condition as a string.
#'
#' @return A formatted SQL condition string or NULL if input is empty.
#' @export
#'
extracondsqlstr <- function(extracond) {
  if (is.null(extracond) || !nzchar(extracond)) {
    return(NULL)
  }
  return(ifelse(grepl("^\\s*\\(", extracond),
    paste0("( ", extracond, " )"),
    extracond
  ))
}

#' Generate Repeat Column SQL Statement
#'
#' This function constructs an SQL condition for repeat samples.
#'
#' @param reps Repeat sample IDs as a string.
#'
#' @return A string representing the SQL condition for repeat samples.
#' @importFrom stringr str_split str_c
#' @export
#'
repsqlstr <- function(reps) {
  if (!nzchar(reps)) {
    return(NULL)
  }

  reps <- gsub("\\s{2,}", "", reps)
  reps <- gsub("[;；，]", ",", reps)
  reps <- gsub("'", "", reps)
  reps_split <- stringr::str_split(reps, ",", simplify = TRUE)

  reps1 <- as.numeric(gsub("[^0-9.-]+", "", reps_split[!grepl("-", reps_split)]))
  repsseq <- stringr::str_split(reps_split[grepl("-", reps_split)], "-")

  # Generate sequences for ranges
  reps2 <- unlist(lapply(repsseq, function(x) {
    ind <- as.numeric(gsub("[^0-9.-]+", "", x))
    seq(ind[1], ind[2])
  }))

  reps <- unique(c(reps1, reps2))
  reps <- na.omit(reps) # Remove NA values

  if (length(reps) == 0) {
    return(NULL)
  }

  return(paste0("(`rep` IN (", paste(reps, collapse = ","), "))"))
}

#' Get SQL Statement for Tax Database
#'
#' This function constructs SQL conditions for filtering based on taxonomy.
#'
#' @param taxs Taxonomy names to limit as a string.
#' @param tbname Database table name.
#'
#' @return A string representing the SQL condition for the taxonomy.
#' @importFrom stringr str_c str_split
#' @export
#'
taxsqlstr <- function(taxs, tbname = "cc") {
  if (length(taxs) > 1) {
    warning("Only the first element will be used!")
    taxs <- taxs[1]
  }

  if (!nzchar(taxs)) {
    return(NULL)
  }

  taxs <- gsub("\\s{2,}", " ", taxs)
  taxs <- gsub("[;；，]", ",", taxs)

  if (!nzchar(taxs)) {
    return(invisible(NULL))
  }

  taxs <- gsub("'", "", taxs)
  taxs <- unique(stringr::str_split(taxs, ",", simplify = TRUE))
  taxs <- taxs[nzchar(taxs)]

  if (tbname == "cc") {
    taxstr1 <- paste(paste0("`tax` LIKE '%", taxs, "%'"), collapse = " OR ")
    taxstr2 <- paste0(
      "(`tax` IN (SELECT `taxonen` FROM `tax` WHERE (`phytaen` LIKE '%", taxs,
      "%' OR `phytacn` LIKE '%", taxs, "%')))"
    )
    taxstr2 <- paste(taxstr2, collapse = " OR ")
    taxstr3 <- paste0(
      "(`tax` IN (SELECT `taxoncn` FROM `tax` WHERE (`phytaen` LIKE '%", taxs,
      "%' OR `phytacn` LIKE '%", taxs, "%')))"
    )
    taxstr3 <- paste(taxstr3, collapse = " OR ")
    return(paste0("(", taxstr1, " OR ", taxstr2, " OR ", taxstr3, ")"))
  }

  if (tbname == "tax") {
    taxstr1 <- paste(paste0("`speciesen` LIKE '%", taxs, "%'"),
      collapse = " OR "
    )
    taxstr2 <- paste(paste0("`speciescn` LIKE '%", taxs, "%'"),
      collapse = " OR "
    )
    taxstr3 <- paste(paste0("`phytaen` LIKE '%", taxs, "%'"),
      collapse = " OR "
    )
    taxstr4 <- paste(paste0("`phytacn` LIKE '%", taxs, "%'"),
      collapse = " OR "
    )
    outstr <- paste0(
      "(",
      paste(taxstr1, taxstr2, taxstr3, taxstr4, sep = " OR "), ")"
    )
    return(outstr)
  }
}

#' Generate SQL Condition for Belongto
#'
#' This function creates a SQL condition based on the "belongto" field.
#'
#' @param belongtos The values for "belongto" as a string.
#'
#' @return A string representing the SQL condition for belongto.
#' @importFrom stringr str_split
#' @export
#'
belongtosqlstr <- function(belongtos) {
  if (!nzchar(belongtos)) {
    return(NULL)
  }

  belongtos <- gsub("[^0-9A-z]", "", belongtos)
  belongtos <- gsub("\\s{2,}", "", belongtos)
  belongtos <- gsub("[;\uff1b\uff0c]", ",", belongtos) # Replace Chinese punctuation
  belongtos <- gsub("'", "", belongtos)

  belongtos <- stringr::str_split(belongtos, ",", simplify = TRUE)
  return(paste0(
    "(`belongto1` IN (",
    paste(shQuote(belongtos), collapse = ", "), "))"
  ))
}

==> cctda_0.0.48.tar.gz: cctda/R/get_ccsqlstr.R <==
#' Get SQL Conditional Statement for ccdb
#'
#' This function constructs an SQL WHERE clause based on provided parameters
#' for querying the cc database.
#'
#' @param daterange Date range as a vector of two dates.
#' @param datelim (deprecated!) Date range as a vector of two dates.
#' @param pids Project IDs as a string.
#' @param sites Sites as a string.
#' @param zlim Water depth range of samples as a numeric vector.
#' @param reps Repeat sample IDs as a string.
#' @param taxs Taxonomy names as a string.
#' @param extracond Extra condition statement as a string.
#'
#' @return A string representing the SQL WHERE clause.
#' @importFrom lubridate today
#' @importFrom stringr str_c str_split
#' @export
#'
get_ccsqlstr <- function(
  daterange = lubridate::today() - c(365, 0),
  pids = "",
  sites = "",
  zlim = c(0, 30),
  taxs = "",
  reps = "",
  extracond = "",
  datelim = NULL
) {
  if (!is.null(datelim)) {
    warning("'datelim' is deprecated; using 'daterange' instead!")
    daterange <- datelim
  }
  # Construct SQL conditions
  datestr <- sprintf(
    "(`date` BETWEEN '%s' AND '%s')",
    daterange[1],
    daterange[2]
  )
  pidstr <- pidsitesqlstr(pids, type = "pid")
  sitestr <- pidsitesqlstr(sites, type = "site")
  zstr <- sprintf("(`depth` >= %d AND `depth` <= %d)", zlim[1], zlim[2])
  taxstr <- taxsqlstr(taxs, tbname = "cc")
  repstr <- repsqlstr(reps)
  extracondstr <- extracondsqlstr(extracond)

  # Collect all conditions
  condstrs <- c(datestr, pidstr, sitestr, zstr, taxstr, repstr, extracondstr)
  condstrs <- condstrs[nzchar(condstrs)] # Remove empty conditions

  if (length(condstrs) == 0) {
    return("")
  }

  return(paste0(" WHERE (", paste(condstrs, collapse = " AND "), ")"))
}

#' Generate Extra Conditional SQL Statement
#'
#' This function formats an extra SQL condition.
#'
#' @param extracond Custom SQL condition as a string.
#'
#' @return A formatted SQL condition string or NULL if input is empty.
#' @export
#'
extracondsqlstr <- function(extracond) {
  if (is.null(extracond) || !nzchar(extracond)) {
    return(NULL)
  }
  return(ifelse(
    grepl("^\\s*\\(", extracond),
    paste0("( ", extracond, " )"),
    extracond
  ))
}

#' Generate Repeat Column SQL Statement
#'
#' This function constructs an SQL condition for repeat samples.
#'
#' @param reps Repeat sample IDs as a string.
#'
#' @return A string representing the SQL condition for repeat samples.
#' @importFrom stringr str_split str_c
#' @importFrom stats na.omit
#' @export
#'
repsqlstr <- function(reps) {
  if (!nzchar(reps)) {
    return(NULL)
  }

  reps <- gsub("\\s{2,}", "", reps)
  reps <- gsub("[;；，]", ",", reps)
  reps <- gsub("'", "", reps)
  reps_split <- stringr::str_split(reps, ",", simplify = TRUE)

  reps1 <- as.numeric(gsub(
    "[^0-9.-]+",
    "",
    reps_split[!grepl("-", reps_split)]
  ))
  repsseq <- stringr::str_split(reps_split[grepl("-", reps_split)], "-")

  # Generate sequences for ranges
  reps2 <- unlist(lapply(repsseq, function(x) {
    ind <- as.numeric(gsub("[^0-9.-]+", "", x))
    seq(ind[1], ind[2])
  }))

  reps <- unique(c(reps1, reps2))
  reps <- stats::na.omit(reps) # Remove NA values

  if (length(reps) == 0) {
    return(NULL)
  }

  return(paste0("(`rep` IN (", paste(reps, collapse = ","), "))"))
}

#' Get SQL Statement for Tax Database
#'
#' This function constructs SQL conditions for filtering based on taxonomy.
#'
#' @param taxs Taxonomy names to limit as a string.
#' @param tbname Database table name.
#'
#' @return A string representing the SQL condition for the taxonomy.
#' @importFrom stringr str_c str_split
#' @export
#'
taxsqlstr <- function(taxs, tbname = "cc") {
  if (length(taxs) > 1) {
    warning("Only the first element will be used!")
    taxs <- taxs[1]
  }

  if (!nzchar(taxs)) {
    return(NULL)
  }

  taxs <- gsub("\\s{2,}", " ", taxs)
  taxs <- gsub("[;；，]", ",", taxs)

  if (!nzchar(taxs)) {
    return(invisible(NULL))
  }

  taxs <- gsub("'", "", taxs)
  taxs <- unique(stringr::str_split(taxs, ",", simplify = TRUE))
  taxs <- taxs[nzchar(taxs)]

  if (tbname == "cc") {
    taxstr1 <- paste(paste0("`tax` LIKE '%", taxs, "%'"), collapse = " OR ")
    taxstr2 <- paste0(
      "(`tax` IN (SELECT `taxonen` FROM `tax` WHERE (`phytaen` LIKE '%",
      taxs,
      "%' OR `phytacn` LIKE '%",
      taxs,
      "%')))"
    )
    taxstr2 <- paste(taxstr2, collapse = " OR ")
    taxstr3 <- paste0(
      "(`tax` IN (SELECT `taxoncn` FROM `tax` WHERE (`phytaen` LIKE '%",
      taxs,
      "%' OR `phytacn` LIKE '%",
      taxs,
      "%')))"
    )
    taxstr3 <- paste(taxstr3, collapse = " OR ")
    return(paste0("(", taxstr1, " OR ", taxstr2, " OR ", taxstr3, ")"))
  }

  if (tbname == "tax") {
    taxstr1 <- paste(
      paste0("`speciesen` LIKE '%", taxs, "%'"),
      collapse = " OR "
    )
    taxstr2 <- paste(
      paste0("`speciescn` LIKE '%", taxs, "%'"),
      collapse = " OR "
    )
    taxstr3 <- paste(paste0("`phytaen` LIKE '%", taxs, "%'"), collapse = " OR ")
    taxstr4 <- paste(paste0("`phytacn` LIKE '%", taxs, "%'"), collapse = " OR ")
    outstr <- paste0(
      "(",
      paste(taxstr1, taxstr2, taxstr3, taxstr4, sep = " OR "),
      ")"
    )
    return(outstr)
  }
}

#' Generate SQL Condition for Belongto
#'
#' This function creates a SQL condition based on the "belongto" field.
#'
#' @param belongtos The values for "belongto" as a string.
#'
#' @return A string representing the SQL condition for belongto.
#' @importFrom stringr str_split
#' @export
#'
belongtosqlstr <- function(belongtos) {
  if (!nzchar(belongtos)) {
    return(NULL)
  }

  belongtos <- gsub("[^0-9A-z]", "", belongtos)
  belongtos <- gsub("\\s{2,}", "", belongtos)
  belongtos <- gsub("[;\uff1b\uff0c]", ",", belongtos) # Replace Chinese punctuation
  belongtos <- gsub("'", "", belongtos)

  belongtos <- stringr::str_split(belongtos, ",", simplify = TRUE)
  return(paste0(
    "(`belongto1` IN (",
    paste(shQuote(belongtos), collapse = ", "),
    "))"
  ))
}
